/**********************************************************************
Copyright (c) 2009 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.datanucleus.store.rdbms.sql.expression;

import java.util.Date;

import org.datanucleus.ClassNameConstants;
import org.datanucleus.exceptions.NucleusException;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;
import org.datanucleus.store.rdbms.sql.SQLStatement;
import org.datanucleus.store.types.JodaLocalDateStringConverter;
import org.joda.time.DateTime;
import org.joda.time.LocalTime;

/**
 * Representation of an JodaTime "LocalTime" literal.
 */
public class JodaLocalTimeLiteral extends StringTemporalExpression implements SQLLiteral
{
    private final LocalTime value;

    /**
     * Constructor for an LocalTime literal with a value.
     * @param stmt the SQL statement
     * @param mapping the mapping
     * @param value the value
     */
    public JodaLocalTimeLiteral(SQLStatement stmt, JavaTypeMapping mapping, Object value, boolean parameter)
    {
        super(stmt, null, mapping);
        this.isParameter = parameter;

        if (value == null)
        {
            this.value = null;
        }
        else if (value instanceof LocalTime)
        {
            this.value = (LocalTime)value;
        }
        else
        {
            throw new NucleusException("Cannot create " + this.getClass().getName() +
                " for value of type " + (value != null ? value.getClass().getName() : null));
        }

        if (mapping.getJavaTypeForDatastoreMapping(0).equals(ClassNameConstants.JAVA_LANG_STRING))
        {
            String str = new JodaLocalDateStringConverter().toString(value);
            delegate = new StringLiteral(stmt, mapping,
                (this.value != null ? str : null), isParameter);
        }
        else
        {
            Date date = new Date();
            date.setTime(((DateTime)value).getMillis());
            delegate = new TemporalLiteral(stmt, mapping,
                (this.value != null ? date : null), isParameter);
        }
    }

    /* (non-Javadoc)
     * @see org.datanucleus.store.rdbms.sql.expression.SQLLiteral#getValue()
     */
    public Object getValue()
    {
        return value;
    }

    /* (non-Javadoc)
     * @see org.datanucleus.store.rdbms.sql.expression.SQLExpression#isParameter()
     */
    @Override
    public boolean isParameter()
    {
        return delegate.isParameter();
    }

    /* (non-Javadoc)
     * @see org.datanucleus.store.rdbms.sql.expression.SQLLiteral#setNotParameter()
     */
    public void setNotParameter()
    {
        ((SQLLiteral)delegate).setNotParameter();
    }
}